@@ -80,6 +80,7 @@ class TestAuthentication(tb.ConnectedTestCase):
         methods = [
             ('trust', None),
             ('reject', None),
+            ('scram-sha-256', 'correctpassword'),
             ('md5', 'correctpassword'),
             ('password', 'correctpassword'),
         ]
@@ -88,27 +89,46 @@ class TestAuthentication(tb.ConnectedTestCase):
 
         create_script = []
         for method, password in methods:
+            if method == 'scram-sha-256' and self.server_version.major < 10:
+                continue
+
+            username = method.replace('-', '_')
+
+            # if this is a SCRAM password, we need to set the encryption method
+            # to "scram-sha-256" in order to properly hash the password
+            if method == 'scram-sha-256':
+                create_script.append(
+                    "SET password_encryption = 'scram-sha-256';"
+                )
+
             create_script.append(
                 'CREATE ROLE {}_user WITH LOGIN{};'.format(
-                    method,
+                    username,
                     ' PASSWORD {!r}'.format(password) if password else ''
                 )
             )
 
+            # to be courteous to the MD5 test, revert back to MD5 after the
+            # scram-sha-256 password is set
+            if method == 'scram-sha-256':
+                create_script.append(
+                    "SET password_encryption = 'md5';"
+                )
+
             if _system != 'Windows':
                 self.cluster.add_hba_entry(
                     type='local',
-                    database='postgres', user='{}_user'.format(method),
+                    database='postgres', user='{}_user'.format(username),
                     auth_method=method)
 
             self.cluster.add_hba_entry(
                 type='host', address=ipaddress.ip_network('127.0.0.0/24'),
-                database='postgres', user='{}_user'.format(method),
+                database='postgres', user='{}_user'.format(username),
                 auth_method=method)
 
             self.cluster.add_hba_entry(
                 type='host', address=ipaddress.ip_network('::1/128'),
-                database='postgres', user='{}_user'.format(method),
+                database='postgres', user='{}_user'.format(username),
                 auth_method=method)
 
         # Put hba changes into effect
@@ -124,13 +144,19 @@ class TestAuthentication(tb.ConnectedTestCase):
         methods = [
             'trust',
             'reject',
+            'scram-sha-256',
             'md5',
             'password',
         ]
 
         drop_script = []
         for method in methods:
-            drop_script.append('DROP ROLE {}_user;'.format(method))
+            if method == 'scram-sha-256' and self.server_version.major < 10:
+                continue
+
+            username = method.replace('-', '_')
+
+            drop_script.append('DROP ROLE {}_user;'.format(username))
 
         drop_script = '\n'.join(drop_script)
         self.loop.run_until_complete(self.con.execute(drop_script))
@@ -189,6 +215,53 @@ class TestAuthentication(tb.ConnectedTestCase):
             await self._try_connect(
                 user='md5_user', password='wrongpassword')
 
+    async def test_auth_password_scram_sha_256(self):
+        # scram is only supported in PostgreSQL 10 and above
+        if self.server_version.major < 10:
+            return
+
+        conn = await self.connect(
+            user='scram_sha_256_user', password='correctpassword')
+        await conn.close()
+
+        with self.assertRaisesRegex(
+                asyncpg.InvalidPasswordError,
+                'password authentication failed for user "scram_sha_256_user"'
+        ):
+            await self._try_connect(
+                user='scram_sha_256_user', password='wrongpassword')
+
+        # various SASL prep tests
+        # first ensure that password are being hashed for SCRAM-SHA-256
+        await self.con.execute("SET password_encryption = 'scram-sha-256';")
+        alter_password = "ALTER ROLE scram_sha_256_user PASSWORD E{!r};"
+        passwords = [
+            'nonascii\u1680space',  # C.1.2
+            'common\u1806nothing',  # B.1
+            'ab\ufb01c',            # normalization
+            'ab\u007fc',            # C.2.1
+            'ab\u206ac',            # C.2.2, C.6
+            'ab\ue000c',            # C.3, C.5
+            'ab\ufdd0c',            # C.4
+            'ab\u2ff0c',            # C.7
+            'ab\u2000c',            # C.8
+            'ab\ue0001',            # C.9
+        ]
+
+        # ensure the passwords that go through SASLprep work
+        for password in passwords:
+            # update the password
+            await self.con.execute(alter_password.format(password))
+            # test to see that passwords are properly SASL prepped
+            conn = await self.connect(
+                user='scram_sha_256_user', password=password)
+            await conn.close()
+
+        alter_password = \
+            "ALTER ROLE scram_sha_256_user PASSWORD 'correctpassword';"
+        await self.con.execute(alter_password)
+        await self.con.execute("SET password_encryption = 'md5';")
+
     async def test_auth_unsupported(self):
         pass
 
@@ -380,6 +453,41 @@ class TestConnectParams(tb.TestCase):
                 'database': 'dbname'})
         },
 
+        {
+            'dsn': 'postgresql://us%40r:p%40ss@h%40st1,h%40st2:543%33/d%62',
+            'result': (
+                [('h@st1', 5432), ('h@st2', 5433)],
+                {
+                    'user': 'us@r',
+                    'password': 'p@ss',
+                    'database': 'db',
+                }
+            )
+        },
+
+        {
+            'dsn': 'postgresql://user:p@ss@host/db',
+            'result': (
+                [('ss@host', 5432)],
+                {
+                    'user': 'user',
+                    'password': 'p',
+                    'database': 'db',
+                }
+            )
+        },
+
+        {
+            'dsn': 'postgresql:///d%62?user=us%40r&host=h%40st&port=543%33',
+            'result': (
+                [('h@st', 5433)],
+                {
+                    'user': 'us@r',
+                    'database': 'db',
+                }
+            )
+        },
+
         {
             'dsn': 'pq:///dbname?host=/unix_sock/test&user=spam',
             'error': (ValueError, 'invalid DSN')
@@ -402,6 +510,36 @@ class TestConnectParams(tb.TestCase):
                 }
             )
         },
+        {
+            'dsn': 'postgres:///db?host=/cloudsql/'
+                   'project:region:instance-name&user=spam',
+            'result': (
+                [os.path.join(
+                    '/cloudsql/project:region:instance-name',
+                    '.s.PGSQL.5432'
+                )], {
+                    'user': 'spam',
+                    'database': 'db'
+                }
+            )
+        },
+        {
+            'dsn': 'postgres:///db?host=127.0.0.1:5432,/cloudsql/'
+                   'project:region:instance-name,localhost:5433&user=spam',
+            'result': (
+                [
+                    ('127.0.0.1', 5432),
+                    os.path.join(
+                        '/cloudsql/project:region:instance-name',
+                        '.s.PGSQL.5432'
+                    ),
+                    ('localhost', 5433)
+                ], {
+                    'user': 'spam',
+                    'database': 'db'
+                }
+            )
+        },
     ]
 
     @contextlib.contextmanager
@@ -796,14 +934,14 @@ class TestConnectParams(tb.TestCase):
     async def test_connect_args_validation(self):
         for val in {-1, 'a', True, False, 0}:
             with self.assertRaisesRegex(ValueError, 'greater than 0'):
-                await asyncpg.connect(command_timeout=val, loop=self.loop)
+                await asyncpg.connect(command_timeout=val)
 
         for arg in {'max_cacheable_statement_size',
                     'max_cached_statement_lifetime',
                     'statement_cache_size'}:
             for val in {None, -1, True, False}:
                 with self.assertRaisesRegex(ValueError, 'greater or equal'):
-                    await asyncpg.connect(**{arg: val}, loop=self.loop)
+                    await asyncpg.connect(**{arg: val})
 
 
 class TestConnection(tb.ConnectedTestCase):
@@ -902,8 +1040,7 @@ class TestConnection(tb.ConnectedTestCase):
         con = await asyncpg.connect(
             port=conn_spec.get('port'),
             database=conn_spec.get('database'),
-            user=conn_spec.get('user'),
-            loop=self.loop)
+            user=conn_spec.get('user'))
         await con.close()
 
 
@@ -1069,7 +1206,7 @@ class TestSSLConnection(tb.ConnectedTestCase):
                 self.assertEqual(await con.fetchval('SELECT 43'), 43)
 
         tasks = [worker() for _ in range(100)]
-        await asyncio.gather(*tasks, loop=self.loop)
+        await asyncio.gather(*tasks)
         await pool.close()
 
 
